@@ -9,7 +9,6 @@ import open3d as o3d
 
 from pydrake.all import Meshcat
 from manipulation.mustard_depth_camera_example import MustardExampleSystem
-from manipulation.meshcat_cpp_utils import draw_open3d_point_cloud
 from manipulation.open3d_utils import *
 
 system = MustardExampleSystem()
@@ -33,7 +32,7 @@ for i in range(3):
     plt.title('Depth image')
 
     point_cloud = system.GetOutputPort(f"camera{i}_point_cloud").Eval(context)
-    pcd = create_open3d_point_cloud(point_cloud)
+    pcd = drake_cloud_to_open3d(point_cloud)
     draw_open3d_point_cloud(meshcat, f"camera{i}_point_cloud", pcd)
 
 plt.show()
